/*
jshint esversion: 6
*/

// Mouse controls

var canvas;
var ctxt;
var fps;
const WIN_SCORE = 5;

var onWinScreen = true;

var scoreBoard = {
    score1: 0,
    score2: 0,
    fontSize: 30,
    x: 5,
    y: 30,
    draw: function () {
        ctxt.font = String(this.fontSize) + "px Arial";
        ctxt.fillStyle = paddle1.color;
        ctxt.fillText(this.score1, this.x, this.y);
        ctxt.fillStyle = paddle2.color;
        ctxt.fillText(this.score2, canvas.width - (this.fontSize / 1.5) * countNumDigits(this.score2), this.y);
    },
    reset: function () {
        this.score1 = 0;
        this.score2 = 0;
    }
};

var paddle1 = {
    x: 0,
    y: 0,
    width: 10,
    height: 100,
    color: "red",
    draw: function () {
        ctxt.fillStyle = this.color;
        ctxt.fillRect(this.x, this.y, this.width, this.height);
    }
};

var paddle2 = {
    x: 0,
    y: 0,
    width: 10,
    height: 100,
    color: "blue",
    draw: function () {
        ctxt.fillStyle = this.color;
        ctxt.fillRect(this.x, this.y, this.width, this.height);
    }
};

var ball = {
    x: 0,
    y: 0,
    speed: 3,
    velX: 0,
    velY: 0,
    r: 5,
    color: "white",
    draw: function () {
        ctxt.fillStyle = this.color;
        ctxt.beginPath();
        ctxt.arc(this.x, this.y, this.r, 0, Math.PI * 2, true);
        ctxt.fill();
    },

    reset: function () {

        // Check if a player has won the game
        if (scoreBoard.score1 >= WIN_SCORE || scoreBoard.score2 >= WIN_SCORE)
            onWinScreen = true;

        this.x = canvas.width / 2;
        this.y = canvas.width / 2;
        this.velX *= -1;
        this.velY = 3 * Math.sign(this.velX);
    }
};

var paddles = [paddle1, paddle2];


window.onload = function () {
    setup();

    setInterval(function () {
        update();
        drawCanvas();
    }, 1000 / fps);
};


function setup() {
    canvas = document.querySelector("#gameCanvas");
    ctxt = canvas.getContext("2d");
    fps = 60;
    ball.x = canvas.width / 2;
    ball.y = canvas.height / 2;
    ball.velX = -ball.speed;
    ball.velY = ball.speed;
    paddle1.x = 50;
    paddle1.y = (canvas.height - paddle1.height) / 2;
    paddle2.x = canvas.width - 50;
    paddle2.y = (canvas.height - paddle1.height) / 2;
    document.addEventListener("mousemove", movePaddlePlayer);
    document.addEventListener("mousedown", handleMouseClick);
}


function update() {

    if (onWinScreen)
        return;

    movePaddleComputer();

    ball.x += ball.velX;
    ball.y += ball.velY;

    // Handle paddle collisions
    var ballRight = ball.x + ball.r;
    var ballLeft = ball.x - ball.r;
    var ballBottom = ball.y + ball.r;
    var ballTop = ball.y - ball.r;

    paddles.forEach(function (paddle) {
        if (ballRight >= paddle.x && ballLeft <= paddle.x + paddle.width && ballBottom >= paddle.y && ballTop <= paddle.y + paddle.height) {
            ball.velX *= -1;

            var deltaY = ball.y - (paddle.y + paddle.height / 2);
            ball.velY = deltaY * 0.35;
        }
    });

    if (ball.x + ball.r >= canvas.width) {
        scoreBoard.score1++; // Must come before ball.reset()
        ball.reset();
    }

    if (ball.x <= 0 + ball.r) {
        scoreBoard.score2++;
        ball.reset();
    }

    if (ball.y + ball.r >= canvas.height)
        ball.velY *= -1;

    if (ball.y - ball.r <= 0 + ball.r)
        ball.velY *= -1;

    paddles.forEach(function (paddle) {
        if (paddle.y + paddle.height > canvas.height)
            paddle.y = canvas.height - paddle.height;

        if (paddle.y < 0)
            paddle.y = 0;
    });
}


function drawCanvas() {

    ctxt.fillStyle = "black";
    ctxt.fillRect(0, 0, canvas.width, canvas.height);

    if (onWinScreen) {
        ctxt.font = "30px Arial";
        ctxt.fillStyle = "white";
        ctxt.fillText("Click to continue", canvas.width / 4, canvas.height / 1.4);
        ctxt.font = "50px Arial";
        if (scoreBoard.score1 > scoreBoard.score2)
            ctxt.fillText("Player 1 won!", canvas.width / 4, canvas.height / 3);
        else if (scoreBoard.score2 > scoreBoard.score1)
            ctxt.fillText("Player 2 won!", canvas.width / 4, canvas.height / 3);
        return;
    }

    paddle1.draw();
    paddle2.draw();
    ball.draw();
    scoreBoard.draw();
}


function calculateMousePos(e) {
    var rect = canvas.getBoundingClientRect();
    var root = document.documentElement;
    var mouseX = e.clientX - rect.left - root.scrollLeft;
    var mouseY = e.clientY - rect.top - root.scrollTop;

    return {
        x: mouseX,
        y: mouseY
    };
}


function handleMouseClick(e) {
    if (onWinScreen) {
        scoreBoard.reset();
        onWinScreen = false;
    }
}


function movePaddlePlayer(e) {
    var mousePos = calculateMousePos(e);
    paddle1.y = mousePos.y - (paddle1.height / 2);
}


function movePaddleComputer() {
    var paddle2YCenter = paddle2.y + (paddle2.height / 2);

    // Moves the paddle to hit the ball. 
    // The multiplying part adds a bit of a delay
    if (paddle2YCenter < ball.y - (paddle2.height * 0.2))
        paddle2.y += ball.speed;
    else if (paddle2YCenter > ball.y + (paddle2.height * 0.2))
        paddle2.y -= ball.speed;
}


function countNumDigits(x) {
    var count = 1;
    while (x >= 10) {
        x /= 10;
        count++;
    }
    return count;
}